'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var URI = 'RemoteData';
/**
 * We have not asked for the data yet.
 */
var NotAsked = function () { return ({ tag: 'NotAsked' }); };
/**
 * We asked for the data but have not received an answer yet.
 */
var Loading = function () { return ({ tag: 'Loading' }); };
/**
 * We recieved a failure response.
 * @param error Anything that represents the failure.
 */
var Failure = function (error) { return ({ tag: 'Failure', error: error }); };
/**
 * We recieved the data successfully.
 * @param data The successful payload from the response.
 */
var Success = function (data) { return ({ tag: 'Success', data: data }); };
var isNotAsked = function (rd) { return rd.tag === 'NotAsked'; };
var isLoading = function (rd) { return rd.tag === 'Loading'; };
var isFailure = function (rd) { return rd.tag === 'Failure'; };
var isSuccess = function (rd) { return rd.tag === 'Success'; };
var SRD = {
    URI: URI,
    map: function (f, fa) { return isSuccess(fa) ? Success(f(fa.data)) : fa; },
    map2: function (f, fa, fb) { return isSuccess(fa)
        ? isSuccess(fb)
            ? Success(f(fa.data, fb.data))
            : fb
        : fa; },
    map3: function (f, fa, fb, fc) { return isSuccess(fa)
        ? isSuccess(fb)
            ? isSuccess(fc)
                ? Success(f(fa.data, fb.data, fc.data))
                : fc
            : fb
        : fa; },
    chain: function (f, fa) { return isSuccess(fa) ? f(fa.data) : fa; },
    bimap: function (failureFn, successFn, rd) { return isSuccess(rd)
        ? Success(successFn(rd.data))
        : isFailure(rd)
            ? Failure(failureFn(rd.error))
            : rd; },
    ap: function (rdFn, rd) { return isSuccess(rd)
        ? isSuccess(rdFn)
            ? Success(rdFn.data(rd.data))
            : rdFn
        : rd; },
    alt: function (def, rd) { return isSuccess(rd) ? rd : def; },
    of: function (a) { return Success(a); },
    equals: function (a, b) { return a.tag === b.tag; },
    isSuccess: isSuccess,
    isFailure: isFailure,
    isLoading: isLoading,
    isNotAsked: isNotAsked
};
// const mapFailure = () => {}
// const unwrap = () => {}
// const match = () => {}
// const promap = () => {}
// const concat = () => {}
// const reduce = () => {}
// const traverse = () => {}
// const extend = () => {}
// const extract = () => {}
// /**
//  * Always returns a NotAsked.
//  */
// const empty = (): NotAsked => NotAsked()
// /**
//  * Always returns a NotAsked.
//  */
// const zero = (): NotAsked => NotAsked()
// const chainRec = () => {}

exports.Failure = Failure;
exports.Loading = Loading;
exports.NotAsked = NotAsked;
exports.Success = Success;
exports.default = SRD;
